@@ -21,34 +21,6 @@
 #include "xstrided_view.hpp"
 #include "xmanipulation.hpp"
 
-namespace xtl
-{
-    template <class CT, class CB>
-    class xoptional;
-
-    template <class... Args>
-    struct at_least_one_xoptional : disjunction<is_xoptional<Args>...>
-    {
-    };
-
-    template <class B, class T1, class T2, XTL_REQUIRES(negation<at_least_one_xoptional<B, T1, T2>>)>
-    inline std::common_type_t<T1, T2> select(const B& cond, const T1& v1, const T2& v2) noexcept
-    {
-        return cond ? v1 : v2;
-    }
-
-    template <class B, class T1, class T2, XTL_REQUIRES(at_least_one_xoptional<B, T1, T2>)>
-    inline common_optional_t<T1, T2> select(const B& cond, const T1& v1, const T2& v2) noexcept
-    {
-        using bool_type = common_optional_t<B>;
-        using return_type = common_optional_t<T1, T2>;
-        bool_type opt_cond(cond);
-        return opt_cond.has_value() ?
-            opt_cond.value() ? return_type(v1) : return_type(v2) :
-            missing<typename return_type::value_type>();
-    }
-}
-
 namespace xt
 {
 
