@@ -9,14 +9,7 @@
 #ifndef XTL_XVARIANT_HPP
 #define XTL_XVARIANT_HPP
 
-#ifndef __cpp_exceptions
-    #define __cpp_exceptions
-    #include "xvariant_impl.hpp"
-    #undef __cpp_exceptions
-#else
-    #include "xvariant_impl.hpp"
-#endif
-
+#include "xvariant_impl.hpp"
 #include "xclosure.hpp"
 #include "xmeta_utils.hpp"
 
